/*
 * Pluginsystem
 * Copyright (C) Thasso Griebel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.java.plugin;

import com.google.inject.*;
import com.google.inject.name.Names;
import org.java.plugin.extension.AllPluginsClassLoader;
import org.java.plugin.extension.GuicyObjectFactory;
import org.java.plugin.extension.GuicyLifecycleHandler;
import org.java.plugin.extension.InstanceFactory;

import org.java.plugin.ObjectFactory;
import org.java.plugin.PluginController;
import org.java.plugin.registry.xml.PluginRegistryImpl;
import org.java.plugin.standard.StandardPathResolver;

/**
 * Guice Module for the PluginManager structure. Creates bindings for
 * the plugins resources. This includes the {@link org.java.plugin.PluginController},
 * the {@link AllPluginsClassLoader} and {@link ClassLoader} in general.
 * The latter is created by a custom provider that returns the this class's ClassLoader.
 *
 * 
 * Author: thasso
 * Date: May 8, 2009
 */
class PluginControllerModule extends AbstractModule{
    @Inject
    private Injector injector;
    @Inject
    private PluginController controller;

    protected void configure() {
        bind(PluginController.class).in(Scopes.SINGLETON);
        bind(ObjectFactory.class).to(GuicyObjectFactory.class);

        bind(org.java.plugin.PathResolver.class).to(StandardPathResolver.class).in(Scopes.SINGLETON);

        bind(org.java.plugin.standard.PluginLifecycleHandler.class).to(GuicyLifecycleHandler.class).in(Scopes.SINGLETON);
        bind(GuicyLifecycleHandler.class).in(Scopes.SINGLETON);

        bind(org.java.plugin.registry.PluginRegistry.class).to(PluginRegistryImpl.class).in(Scopes.SINGLETON);
        /*
        Make the AllPluginsClassLoader singleton and instanciated when the
        injector gets created
         */
        bind(Key.get(ClassLoader.class, Names.named("PluginsClassLoader"))).toInstance(AllPluginsClassLoader.getInstance());
        bind(AllPluginsClassLoader.class).in(Scopes.SINGLETON);
        
        bind(ClassLoader.class).toProvider(new Provider<ClassLoader>() {
            public ClassLoader get() {
                return getClass().getClassLoader();
            }
        });

        requestStaticInjection(InstanceFactory.class);
    }
}
