/*
 * Pluginsystem
 * Copyright (C) Thasso Griebel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.java.plugin.extension;

import org.java.plugin.standard.StandardPluginLifecycleHandler;
import org.java.plugin.standard.StandardObjectFactory;
import org.java.plugin.Plugin;
import org.java.plugin.PluginLifecycleException;
import org.java.plugin.util.ExtendedProperties;
import org.java.plugin.registry.PluginDescriptor;
import com.google.inject.*;


/**
 * Author: thasso
 * Date: May 8, 2009
 */
public class GuicyLifecycleHandler extends StandardPluginLifecycleHandler{
    private Injector injector;

    @Inject
    public GuicyLifecycleHandler(Injector injector){
        super();
        this.injector = injector;
    }

    @Override
    protected Plugin createPluginInstance(PluginDescriptor descr) throws PluginLifecycleException {
        Class<?> pluginClass = getPluginClass(descr);
        Plugin p = (Plugin) injector.getInstance(pluginClass);
        if(p != null){
            return p;
        }        
        try {
            return (Plugin) pluginClass.newInstance();
        } catch (InstantiationException ie) {
            throw new PluginLifecycleException(
                    StandardObjectFactory.PACKAGE_NAME,
                    "pluginClassInstantiationFailed", descr.getId(), ie); //$NON-NLS-1$
        } catch (IllegalAccessException iae) {
            throw new PluginLifecycleException(
                    StandardObjectFactory.PACKAGE_NAME,
                    "pluginClassInstantiationFailed", descr.getId(), iae); //$NON-NLS-1$
        }
    }

    public Class getPluginClass(PluginDescriptor descr) throws PluginLifecycleException {
        String className = descr.getPluginClassName();
        try {
            return getPluginManager().getPluginClassLoader(descr).loadClass(className);            
        } catch (ClassNotFoundException cnfe) {
            throw new PluginLifecycleException(
                    StandardObjectFactory.PACKAGE_NAME,
                    "pluginClassNotFound", className, cnfe); //$NON-NLS-1$
        }
    }

    @Override
    public void configure(ExtendedProperties config) {
        super.configure(config);
    }
}
