/*
 * Pluginsystem
 * Copyright (C) Thasso Griebel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.java.plugin.extension;

import org.java.plugin.standard.StandardObjectFactory;
import org.java.plugin.standard.PluginLifecycleHandler;
import org.java.plugin.standard.StandardPluginManager;
import org.java.plugin.util.ExtendedProperties;
import org.java.plugin.registry.PluginRegistry;
import org.java.plugin.PathResolver;
import org.java.plugin.PluginManager;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.google.inject.Injector;
import com.google.inject.Inject;

/**
 * Author: thasso
 * Date: May 8, 2009
 */
public class GuicyObjectFactory extends StandardObjectFactory{
    private Injector injector;
    
    @Inject
    public GuicyObjectFactory(Injector injector){
        super();
        this.injector = injector;
    }    

    protected Object createClassInstance(final String className)
            throws InstantiationException, IllegalAccessException,
            ClassNotFoundException {
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        if (cl != null) {
            try {
                Class clazz = cl.loadClass(className);
                Object result = injector.getInstance(clazz);
                if(result != null){
                    return result;
                }else{
                    return super.createClassInstance(className);
                }                
            } catch (ClassNotFoundException cnfe) {
                // ignore
            }
        }
        cl = getClass().getClassLoader();
        if (cl != null) {
            try {
                Class clazz = cl.loadClass(className);
                Object result = injector.getInstance(clazz);
                if(result != null){
                    return result;
                }else{
                    return super.createClassInstance(className);
                }                                
            } catch (ClassNotFoundException cnfe) {
                // ignore
            }
        }

        cl = ClassLoader.getSystemClassLoader();
        try {
            Class clazz = cl.loadClass(className);
            Object result = injector.getInstance(clazz);
            if(result != null){
                return result;
            }else{
                return super.createClassInstance(className);
            }
        } catch (ClassNotFoundException cnfe) {
            // ignore
        }
        return null;
    }

    @Override
    public PluginRegistry createRegistry() {
        PluginRegistry registry = injector.getInstance(PluginRegistry.class);
        if(registry != null){
            registry.configure(config);
            return registry;
        }
        return super.createRegistry();
    }

    @Override
    public PathResolver createPathResolver() {
        PathResolver resolver = injector.getInstance(PathResolver.class);
        if(resolver != null){
            try {
                resolver.configure(config);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return resolver;
        }
        return super.createPathResolver();
    }

    @Override
    protected PluginLifecycleHandler createLifecycleHandler() {
        PluginLifecycleHandler lifecycleHandler = injector.getInstance(PluginLifecycleHandler.class);
        if(lifecycleHandler != null){
            lifecycleHandler.configure(config.getSubset("org.java.plugin.standard.StandardPluginLifecycleHandler."));
            return lifecycleHandler;
        }
        return super.createLifecycleHandler();
    }

    @Override
    public PluginManager createManager(PluginRegistry registry, PathResolver pathResolver) {
        return super.createManager(registry, pathResolver);    //To change body of overridden methods use File | Settings | File Templates.
    }
}
