/*
 * Pluginsystem
 * Copyright (C) Thasso Griebel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.java.plugin.extension.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Used to mark classes as extensions. The extended ExtensionPoint
 * must provide a parameter called 'class' !
 * 
 * The annotated class's name is taken as class parameter (you do not have to specify this parameter manually)
 *
 *
 */
@Inherited
@Retention(value = RetentionPolicy.RUNTIME)
@Target(value={ElementType.TYPE})
public @interface Extension {
    /**
     * Specify the unique id of the extension point
     * @return uniqueID extension point
     */
    String value();

    /**
     * Specify a extension id. The default id will be thas full class name of the annotated class
     * @return
     */
	String id() default "";

    /**
     * Optional set of parameters 
     *
     * @return parameter list
     */
	Parameter[] parameter() default {};
}
