/*
 * Pluginsystem
 * Copyright (C) Thasso Griebel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.java.plugin.extension.annotations.scanner;

import org.java.plugin.extension.AnnotatedExtension;
import org.java.plugin.registry.Extension;
import org.java.plugin.registry.PluginDescriptor;
import org.objectweb.asm.AnnotationVisitor;

import java.util.ArrayList;
import java.util.List;

/**
 * Visits @Extension annotations and reads the values to create a new {@link org.java.plugin.registry.Extension}.
 * The annotated class's name is taken as class parameter (you do not have to specify this parameter manually) 
 *
 * @author Thasso Griebel (thasso.griebel@gmail.com)
 * @see org.java.plugin.registry.Extension
 * @see org.java.plugin.extension.annotations.Extension
 */
public class ExtensionVisitor implements AnnotationVisitor{
    /**
     * The annotatated class
     */
	private String annotatedClass;
    /**
     * custom id
     */
	private String id;
    /**
     * unique point id
     */
    private String uniqueID;
    /**
     * stores parameters
     */
	private List<ParameterContainer> parameters;
    /**
     * the underlying extension list
     */
	private List<Extension> extensionList;
    /**
     * the plugin descriptor
     */
	private PluginDescriptor pluginDescriptor;
	
	public ExtensionVisitor(PluginDescriptor pluginDescriptor, String annotatedClass, List<Extension> extensionList){
		this.annotatedClass = annotatedClass;
		this.extensionList = extensionList;
		this.pluginDescriptor = pluginDescriptor;
        this.id = annotatedClass;
	}
	
	public void visit(String name, Object value) {
		if(name.equals("id")) id = value.toString();
		else if(name.equals("value")) uniqueID = value.toString();

	}
	public AnnotationVisitor visitArray(String name) {
		if(name.equals("parameter")){
			if(parameters == null) parameters = new ArrayList<ParameterContainer>();
			return new ParameterVisitor(parameters);
		}
		return null;
	}

	public void visitEnd() {
		// create and add the extension
        // translate unique id to plugin and extension point
        String[] split = uniqueID.split("@");

        // create class parameter !
        ParameterContainer classParameter = new ParameterContainer("class", annotatedClass, String.class);
        if(parameters == null) parameters = new ArrayList<ParameterContainer>();
		parameters.add(classParameter);

		extensionList.add(new AnnotatedExtension(pluginDescriptor, id, split[0], split[1], parameters));
	}
	
	public void visitEnum(String name, String desc, String value) {}
	public AnnotationVisitor visitAnnotation(String name, String desc) {return null;}
}
