/*
 * Pluginsystem
 * Copyright (C) Thasso Griebel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.java.plugin.extension.annotations.scanner;

import java.util.List;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Type;

/**
 * Visits @Parameter annotations and created new {@link org.java.plugin.extension.annotations.scanner.ParameterContainer}
 * 
 * @author Thasso Griebel (thasso.griebel@gmail.com)
 */
public class ParameterVisitor implements AnnotationVisitor{
	private String key;
	private String value;
	private Class<?> type = String.class;
	private List<ParameterContainer> parameterList;
	
	public ParameterVisitor(List<ParameterContainer> parameters) {
		this.parameterList = parameters;
	}

	public void visit(String name, Object value) {
		if(name.equals("id")) key = value.toString();
		if(name.equals("value")) this.value = value.toString();
		if(name.equals("type")){
			Type t = (Type) value;			
			try {
				type = Class.forName(t.getClassName());
			} catch (ClassNotFoundException e) {
				type = String.class;
			}
		}
	}

	public AnnotationVisitor visitAnnotation(String name, String desc) {
		return new ParameterVisitor(parameterList);
	}
	public void visitEnd() {
		if(key != null && value != null){
			parameterList.add(new ParameterContainer(key, value, type));
		}
	}
	public AnnotationVisitor visitArray(String name) {return null;}
	public void visitEnum(String name, String desc, String value) {}
}
