/*
 * Pluginsystem
 * Copyright (C) Thasso Griebel
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.java.plugin.extension.util;

import com.google.inject.matcher.Matcher;
import com.google.inject.matcher.AbstractMatcher;
import com.google.inject.TypeLiteral;

import java.io.Serializable;

/**
 * @author Thasso Griebel (thasso.griebel@gmail.com)
 */
public class TypeMatchers {

    public static Matcher<? super TypeLiteral<?>> superclassesOf(TypeLiteral<?> clazz){
        return new SuperclassesOf(clazz);
    }

    private static class SuperclassesOf extends AbstractMatcher<TypeLiteral<?>> implements Serializable {
        private final TypeLiteral<?> subclass;

        public SuperclassesOf(TypeLiteral<?> subclass) {
            if(subclass == null) throw new NullPointerException();
            this.subclass = subclass;
        }

        @Override public boolean equals(Object other) {
          return other instanceof SuperclassesOf
              && ((SuperclassesOf) other).subclass.equals(subclass);
        }

        @Override public int hashCode() {
          return 37 * subclass.hashCode();
        }

        @Override public String toString() {
          return "superclassesOf(" + subclass.getRawType().getSimpleName() + ".class)";
        }

        private static final long serialVersionUID = 0;

        public boolean matches(TypeLiteral<?> t) {
            return t.getRawType().isAssignableFrom(subclass.getRawType());
        }
    }
}
